/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#ifndef BALLONACTIVITY_H
#define BALLONACTIVITY_H

//#include "oftracker.h"
#include "mhitracker.h"
#include "SimpleShapeDrawer.h"
#include "oscinout.h"
#include "sdlcontroller.h"
#include "puredatapatch.h"
#include "cameracontroller.h"

class CBallonActivity : public CProcessImage2
{
public:
	enum BallonActivityStage { BALLON_BW= 1, BALLON_COLOR, BALLON_BLOWUP, BALLON_BLOWUP_COLOR, FOO_LAST };

	CBallonActivity();
	virtual ~CBallonActivity(void);
	
	void Run();
	void Finish();

	BallonActivityStage GetCurrentStage() { return m_currentStage; }
	BallonActivityStage SetPreviousStage() 
	{
		if (m_currentStage> 1) m_currentStage= (BallonActivityStage) (m_currentStage - 1);
		return m_currentStage;
	}
	BallonActivityStage SetNextStage() 
	{
		if (m_currentStage< FOO_LAST-1) m_currentStage= (BallonActivityStage) (m_currentStage + 1);
		return m_currentStage;
	}
	
	void SetBackgroundColour (unsigned char r, unsigned char g, unsigned char b, unsigned char a);
	void SetDrawingColour (unsigned char r, unsigned char g, unsigned char b, unsigned char a);

	void SetSmoothing (float smoothing);
	float GetSmoothing () const { return (pow(10.0f, m_smoothing) - 1.0f) / 9.0f; }

	void SetBaseFreq (float freq);
	float GetBaseFreq () const { return m_baseFreq; }

	float GetFreqRange () const { return m_freqRange; }
	void SetFreqRange (float range);

	// 1..100
	void SetSensitivy (float sensitivy) { m_sensitivy= pow (10.0f, sensitivy / 25.0f); }
	float GetSensitivy () const { return log10(m_sensitivy) * 25.0f; }

	void SetAutoSensitivy (bool value) { m_autoSensitivy= value; }
	bool GetAutoSensitivy () const { return m_autoSensitivy; }

	void GetBackgroundColour (unsigned char& r, unsigned char& g, unsigned char& b, unsigned char& a) const {
		r= (unsigned char) (m_backgroundColour >> 24);
		g= (unsigned char) (m_backgroundColour >> 16);
		b= (unsigned char) (m_backgroundColour >> 8);
		a= (unsigned char) m_backgroundColour;
	}
	void GetDrawingColour (unsigned char& r, unsigned char& g, unsigned char& b, unsigned char& a) const {
		r= (unsigned char) (m_drawingColour >> 24);
		g= (unsigned char) (m_drawingColour >> 16);
		b= (unsigned char) (m_drawingColour >> 8);
		a= (unsigned char) m_drawingColour;
	}

	// Color-space conversion
	void rgba2hsv (Uint32 rgba, float &h, float &s, float &v);
	Uint32 hsv2rgba (float h, float s, float v);

	CSimpleShapeDrawer* getSimpleShapeDrawer() { return &m_simpleShapeDrawer; }
protected:
	virtual void ProcessImage (IplImage *pImage);
		
private:	
	enum { MOTION_HISTORY_SIZE= 15 };
	_int64 GetTStamp (void) const;
	float ComputeRealMotion (CIplImage& image);
	bool MotionDetected(float motion);
	float ComputeMotionXSensitivy (float scaled_motion);
	
	void PatchSend (float motion, float size);	
	
	// Attributes
	CSDLBackgroundDrawer m_sdlDrawer;
	CPureDataPatch m_patch;
	COscOut m_oscOut;	
//	COfTracker* m_pOfTracker;
	CMhiTracker m_pMhiTracker;
	CCameraProcessor m_camProcessor;

	// Parameters
	float m_smoothing;
	float m_baseFreq;
	float m_freqRange;
	bool m_autoSensitivy;
	float m_sensitivy;
	Uint32 m_backgroundColour, m_drawingColour;
	float m_motionThreshold;
	BallonActivityStage m_currentStage;
	
	CSimpleShapeDrawer m_simpleShapeDrawer;
	_int64 m_currentTStamp, m_lastTStamp, m_lastMotionTStamp;	
	float m_motion_history[MOTION_HISTORY_SIZE];
	int m_motion_history_index;
	float m_prevMotion;
	float m_maxArea;

	// Invitation management -> TODO: implement as a class 
	_int64 m_lastInvitationEndTStamp, m_invitationStartTStamp;
	bool m_invitationFired;
	int m_invitationStage;

	int m_invitationWaitTime, m_invitationDuration;
	bool m_firstTime;
};

#endif