/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

#ifndef _BALLONCONFIGURATION_H_
#define _BALLONCONFIGURATION_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/mdi.h"
#include "wx/clrpicker.h"
////@end includes
#include <wx/slider.h>
#include <wx/stattext.h>
#include <wx/checkbox.h>
#include <wx/timer.h>
#include "ballonactivity.h"

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxColourPickerCtrl;
////@end forward declarations

class CSimpleShapeDrawer;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_BALLONCONFIGURATION 10008
#define ID_PANEL1 10069
#define ID_CHECKBOX_AUTOSENSITIVY 10013
#define ID_SLIDER_SENSITIVY 10001
#define ID_PANEL 10020
#define ID_BUTTON 10014
#define ID_TEXTCTRL 10016
#define ID_BUTTON1 10015
#define ID_SLIDER_SMOOTHING 10009
#define ID_SLIDER_BASE_FREQ 10010
#define ID_SLIDER_FREQ_RANGE 10000
#define ID_COLOURCTRL_BACKGROUND 10011
#define ID_COLOURCTRL_DRAWING 10012
#define SYMBOL_CBALLONCONFIGURATION_STYLE wxCAPTION|wxSYSTEM_MENU|wxCLOSE_BOX|wxTAB_TRAVERSAL
#define SYMBOL_CBALLONCONFIGURATION_TITLE _("Ballon Configuration")
#define SYMBOL_CBALLONCONFIGURATION_IDNAME ID_BALLONCONFIGURATION
#define SYMBOL_CBALLONCONFIGURATION_SIZE wxSize(400, 300)
#define SYMBOL_CBALLONCONFIGURATION_POSITION wxDefaultPosition
////@end control identifiers

#define TIMER_ID 12345

/*!
 * CBallonConfiguration class declaration
 */

class CBallonConfiguration: public wxMDIChildFrame
{    
    DECLARE_CLASS( CBallonConfiguration )
    DECLARE_EVENT_TABLE()
public:
	// Singleton
	static CBallonConfiguration* getInstance(wxMDIParentFrame* parent, wxWindowID id = SYMBOL_CBALLONCONFIGURATION_IDNAME, const wxString& caption = SYMBOL_CBALLONCONFIGURATION_TITLE, const wxPoint& pos = SYMBOL_CBALLONCONFIGURATION_POSITION, const wxSize& size = SYMBOL_CBALLONCONFIGURATION_SIZE, long style = SYMBOL_CBALLONCONFIGURATION_STYLE );
	
private:
	static CBallonConfiguration* g_single;

    /// Constructors
    CBallonConfiguration();
    CBallonConfiguration( wxMDIParentFrame* parent, wxWindowID id = SYMBOL_CBALLONCONFIGURATION_IDNAME, const wxString& caption = SYMBOL_CBALLONCONFIGURATION_TITLE, const wxPoint& pos = SYMBOL_CBALLONCONFIGURATION_POSITION, const wxSize& size = SYMBOL_CBALLONCONFIGURATION_SIZE, long style = SYMBOL_CBALLONCONFIGURATION_STYLE );

    bool Create( wxMDIParentFrame* parent, wxWindowID id = SYMBOL_CBALLONCONFIGURATION_IDNAME, const wxString& caption = SYMBOL_CBALLONCONFIGURATION_TITLE, const wxPoint& pos = SYMBOL_CBALLONCONFIGURATION_POSITION, const wxSize& size = SYMBOL_CBALLONCONFIGURATION_SIZE, long style = SYMBOL_CBALLONCONFIGURATION_STYLE );
 
    /// Destructor
    ~CBallonConfiguration();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CBallonConfiguration event handler declarations

    /// wxEVT_CLOSE_WINDOW event handler for ID_BALLONCONFIGURATION
    void OnCloseWindow( wxCloseEvent& event );

    /// wxEVT_DESTROY event handler for ID_BALLONCONFIGURATION
    void OnDestroy( wxWindowDestroyEvent& event );

    /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX_AUTOSENSITIVY
    void OnCheckboxAutosensitivyClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER_SENSITIVY
    void OnSliderSensitivyUpdated( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON
    void OnButtonPreviousStageClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON1
    void OnButtonNextStageClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER_SMOOTHING
    void OnSliderSmoothingUpdated( wxCommandEvent& event );

    /// wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER_BASE_FREQ
    void OnSliderBaseFreqUpdated( wxCommandEvent& event );

    /// wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER_FREQ_RANGE
    void OnSliderFreqRangeUpdated( wxCommandEvent& event );

    /// wxEVT_COLOURPICKER_CHANGED event handler for ID_COLOURCTRL_BACKGROUND
    void OnColourctrlBackgroundColourPickerChanged( wxColourPickerEvent& event );

    /// wxEVT_COLOURPICKER_CHANGED event handler for ID_COLOURCTRL_DRAWING
    void OnColourctrlDrawingColourPickerChanged( wxColourPickerEvent& event );

////@end CBallonConfiguration event handler declarations

	void OnTimer(wxTimerEvent& event);


////@begin CBallonConfiguration member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CBallonConfiguration member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CBallonConfiguration member variables
    wxCheckBox* m_chkAutoSensitivy;
    wxSlider* m_sldSensitivy;
    wxStaticText* m_txtSensitivy;
    wxButton* m_btnPrevStage;
    wxTextCtrl* m_txtCurrentStage;
    wxButton* m_btnNextStage;
    wxSlider* m_sldSmoothing;
    wxStaticText* m_txtSmoothing;
    wxSlider* m_sldBaseFreq;
    wxStaticText* m_txtBaseFrequency;
    wxSlider* m_sldFreqRange;
    wxStaticText* m_txtFreqRange;
    wxColourPickerCtrl* m_clrctrlBackground;
    wxColourPickerCtrl* m_clrctrlDrawing;
////@end CBallonConfiguration member variables
private:
	void UpdateShape();
	void UpdateStage();

	CBallonActivity m_activity;
//	CSimpleShapeDrawer* m_simpleShapeDrawer;
	wxTimer m_timer;

};

#endif
    // _BALLONCONFIGURATION_H_
