/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "camdialog.h"
#include "cameracontroller.h"

////@begin XPM images
////@end XPM images


/*!
 * CCamDialog type definition
 */

IMPLEMENT_CLASS( CCamDialog, wxMDIChildFrame )

// FPS refresh
DECLARE_EVENT_TYPE(wxEVT_SET_FPS, -1)
DEFINE_EVENT_TYPE(wxEVT_SET_FPS)

/*!
 * CCamDialog event table definition
 */

BEGIN_EVENT_TABLE( CCamDialog, wxMDIChildFrame )

////@begin CCamDialog event table entries
    EVT_CLOSE( CCamDialog::OnCloseWindow )

////@end CCamDialog event table entries
	EVT_COMMAND  (wxID_ANY, wxEVT_SET_FPS, CCamDialog::OnSetFPS)
END_EVENT_TABLE()


/*!
 * CCamDialog constructors
 */

CCamDialog::CCamDialog()
{
    Init();
}

CCamDialog::CCamDialog( wxMDIParentFrame* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create( parent, id, caption, pos, size, style );
}


/*!
 * CCamDialog creator
 */

bool CCamDialog::Create( wxMDIParentFrame* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CCamDialog creation
    wxMDIChildFrame::Create( parent, id, caption, pos, size, style );

    CreateControls();
////@end CCamDialog creation
    return true;
}


/*!
 * CCamDialog destructor
 */

CCamDialog::~CCamDialog()
{
	CCameraController::getInstance()->UnregisterImageDrawer (m_camWindow);
////@begin CCamDialog destruction
////@end CCamDialog destruction
}


/*!
 * Member initialisation
 */

void CCamDialog::Init()
{
////@begin CCamDialog member initialisation
    m_statusBar = NULL;
    m_camWindow = NULL;
////@end CCamDialog member initialisation
}


/*!
 * Control creation for CCamDialog
 */

void CCamDialog::CreateControls()
{    
////@begin CCamDialog content construction
    CCamDialog* itemMDIChildFrame1 = this;

    m_statusBar = new wxStatusBar;
    m_statusBar->Create( itemMDIChildFrame1, ID_STATUSBAR, wxST_SIZEGRIP|wxNO_BORDER );
    m_statusBar->SetFieldsCount(1);
    itemMDIChildFrame1->SetStatusBar(m_statusBar);

    m_camWindow = new CCamWindow2;
    m_camWindow->Create( itemMDIChildFrame1, ID_PANEL_CAMWIN, wxDefaultPosition, wxSize(320, 240), wxSUNKEN_BORDER );

////@end CCamDialog content construction

	m_camWindow->SetReziseParent(true);
	CCameraController::getInstance()->RegisterImageDrawer (m_camWindow);
}


/*!
 * Should we show tooltips?
 */

bool CCamDialog::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CCamDialog::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CCamDialog bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CCamDialog bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CCamDialog::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CCamDialog icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CCamDialog icon retrieval
}


/*!
 * wxEVT_COMMAND_MENU_SELECTED event handler for ID_MENU_CAMERA_SETTINGS
 */
#if 0
void CCamDialog::OnMenuCameraSettingsClick( wxCommandEvent& event )
{

	m_mutex.Lock();
	if (m_pCamera && m_pCamera->HasSettingsDialog())
		m_pCamera->ShowSettingsDialog();
	m_mutex.Unlock();
	event.Skip(false);

}
#endif

/*!
 * wxEVT_UPDATE_UI event handler for ID_MENU_CAMERA_SETTINGS
 */

/*
void CCamDialog::OnMenuCameraSettingsUpdate( wxUpdateUIEvent& event )
{
	if (m_pCamera && m_pCamera->HasSettingsDialog())
		event.Enable(true);
	else
		event.Enable(false);

    event.Skip(false);
}*/

void CCamDialog::SetFPS (long value, CCamDialog::EFPSCondition cond)
{
	wxString text;
	switch (cond)
	{
	case CCamDialog::GOOD:
		text.Printf (_(" %li fps (good)"), value);
		break;
	case CCamDialog::FAIR:
		text.Printf (_(" %li fps (could be improved)"), value);
		break;
	case CCamDialog::POOR:
		text.Printf (_(" %li fps (too low)"), value);
		break;
	default:	
		assert (false);
	}

	// Send by a messege (this method is called from other thread)
	wxCommandEvent event(wxEVT_SET_FPS);
	event.SetString (text);
	
	wxPostEvent(this, event);
}

void CCamDialog::OnSetFPS( wxCommandEvent &event )
{
	m_statusBar->SetStatusText(event.GetString());	
}

#if 0
void CCamDialog::DrawCam (IplImage* pImg) 
{ 
	long fps= -1.0f;

	m_pCamWindow->DrawCam(pImg); 

	/*
	m_mutex.Lock();
	if (m_pCamera)
	{	
		fps= (long)(m_pCamera->GetRealFrameRate() + 0.5);
	}
	m_mutex.Unlock();

	if (fps>= 0.0f) SetFPS (fps, CCamDialog::GOOD);
	*/
}
#endif

/*
void CCamDialog::SetCamera (CCamera* pCamera) 
{ 
	m_mutex.Lock();
	m_pCamera= pCamera; 
	m_mutex.Unlock();
}*/


/*!
 * wxEVT_CLOSE_WINDOW event handler for ID_CCAMDIALOG
 */

void CCamDialog::OnCloseWindow( wxCloseEvent& event )
{
	//CCameraController::getInstance()->UnregisterImageDrawer (m_camWindow);
	Destroy();
    event.Skip(false);
}

