/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#pragma once


// TO DELETE: code, with modifications, moved to "ValueRange.h"

#include <math.h>
#include <assert.h>
#include "exceptions.h"

//
// This class is intended for helping mapping between linear to/from log/exp scales.
// The following function is used:
//
//  y= A*e^(x*B)-A  (defined for x>= 0)
//
// Where A is the "speed" constant (for large values the above function grows faster)
// For convenience A=e^a where "a" is a parameter of the class and can be either
// positive or negative.
//
// This function go through the point (0,0) and other point (p_x, p_y) determined
// by the user (so p_x> 0 and p_y> 0). This two points determines the value of B:
//  
//  B= (1/x) * ln ((y+A)/A)
//
// Usually p_x=p_y=max_range_value so the range between linear and the scaled version 
// does not change, only how the function grows inside it. Finally, the inverse is:
//
//  x= (1/B) * ln ((y+A)/A)
//
// So x can be viewed as the input parameter of a exp function or as the result of
// a log function and y as a parameter of a log function or as the result of a exp func.
//


class CLogExpRange
{
public:
	CLogExpRange()
	{
		setParams (1.0f, 1.0f);
	}

	CLogExpRange(float x, float y, float a= 0)
	{
		setParams (x, y, a);
	}

	void setParams (float x, float y, float a= 0)
	{
		m_A= exp (a);

		if (x<= 0 || y<= 0)
		{
			assert (false);
			throw ValueOutOfRangeException (_T("CLogExpRange"));
		}

		//m_x= x;

		m_B= (1.0f/x) * logf ((y+m_A)/m_A);
	}
/*
	void setP (float x, float y)
	{
		if (x<= 0 || y<= 0)
		{
			assert (false);
			throw ValueOutOfRangeException (_T("CLogExpRange"));
		}

		m_x= x;

		m_B= (1.0f/x) * logf ((y+m_A)/m_A);
	}

	void setSpeed (float a)
	{
		float y= ToExp (m_x);	// Recover y point

		m_A= exp(a);
		setP (m_x, y);        
	}
*/
	float ToExp (float x)
	{
		return m_A * exp (x*m_B) - m_A;
	}

	float ToLog (float y)
	{
		return (1.0f/m_B) * logf ((y+m_A)/m_A);
	}
	
private:
	float m_A;
	float m_B;
//	float m_x;
};
