/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     26/04/2010 11:05:03
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#ifndef _VIRTUALINSTRUMENTCONFIG_H_
#define _VIRTUALINSTRUMENTCONFIG_H_


/*!
 * Includes
 */

////@begin includes
#include "wx/mdi.h"
#include "wx/gbsizer.h"
#include "wx/spinctrl.h"
////@end includes
#include "virtualinstrumentactivity.h"

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxSpinCtrl;
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define ID_CVIRTUALINSTRUMENTCONFIG 10076
#define ID_PANEL2 10077
#define ID_SLIDER_SENSOR_SENSITIVITY 10078
#define ID_TEXTCTRL_SENSOR_SENSITIVITY 10079
#define ID_CHECKBOX_AUTO_DETECT_MOTION_AREA 10080
#define ID_CHOICE_MUSIC_SCORE 10081
#define ID_SPINCTRL_NOTE_DURATION 10082
#define ID_CHECKBOX_MUTE_PREVIOUS_NOTE 10083
#define ID_CHOICE_MUSICAL_INSTRUMENT 10084
#define SYMBOL_CVIRTUALINSTRUMENTCONFIG_STYLE wxCAPTION|wxSYSTEM_MENU|wxCLOSE_BOX
#define SYMBOL_CVIRTUALINSTRUMENTCONFIG_TITLE _("Virtual Instrument")
#define SYMBOL_CVIRTUALINSTRUMENTCONFIG_IDNAME ID_CVIRTUALINSTRUMENTCONFIG
#define SYMBOL_CVIRTUALINSTRUMENTCONFIG_SIZE wxDefaultSize
#define SYMBOL_CVIRTUALINSTRUMENTCONFIG_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CVirtualInstrumentConfig class declaration
 */

class CVirtualInstrumentConfig: public wxMDIChildFrame
{    
    DECLARE_CLASS( CVirtualInstrumentConfig )
    DECLARE_EVENT_TABLE()

public:
	// Singleton
	static CVirtualInstrumentConfig* getInstance( wxMDIParentFrame* parent, wxWindowID id = SYMBOL_CVIRTUALINSTRUMENTCONFIG_IDNAME, const wxString& caption = SYMBOL_CVIRTUALINSTRUMENTCONFIG_TITLE, const wxPoint& pos = SYMBOL_CVIRTUALINSTRUMENTCONFIG_POSITION, const wxSize& size = SYMBOL_CVIRTUALINSTRUMENTCONFIG_SIZE, long style = SYMBOL_CVIRTUALINSTRUMENTCONFIG_STYLE );
	
private:
	static CVirtualInstrumentConfig* g_single;

    /// Constructors
    CVirtualInstrumentConfig();
    CVirtualInstrumentConfig( wxMDIParentFrame* parent, wxWindowID id = SYMBOL_CVIRTUALINSTRUMENTCONFIG_IDNAME, const wxString& caption = SYMBOL_CVIRTUALINSTRUMENTCONFIG_TITLE, const wxPoint& pos = SYMBOL_CVIRTUALINSTRUMENTCONFIG_POSITION, const wxSize& size = SYMBOL_CVIRTUALINSTRUMENTCONFIG_SIZE, long style = SYMBOL_CVIRTUALINSTRUMENTCONFIG_STYLE );

    bool Create( wxMDIParentFrame* parent, wxWindowID id = SYMBOL_CVIRTUALINSTRUMENTCONFIG_IDNAME, const wxString& caption = SYMBOL_CVIRTUALINSTRUMENTCONFIG_TITLE, const wxPoint& pos = SYMBOL_CVIRTUALINSTRUMENTCONFIG_POSITION, const wxSize& size = SYMBOL_CVIRTUALINSTRUMENTCONFIG_SIZE, long style = SYMBOL_CVIRTUALINSTRUMENTCONFIG_STYLE );

    /// Destructor
    ~CVirtualInstrumentConfig();

private:
    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CVirtualInstrumentConfig event handler declarations

    /// wxEVT_COMMAND_SLIDER_UPDATED event handler for ID_SLIDER_SENSOR_SENSITIVITY
    void OnSliderSensorSensitivityUpdated( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX_AUTO_DETECT_MOTION_AREA
    void OnCheckboxAutoDetectMotionAreaClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_MUSIC_SCORE
    void OnChoiceMusicScoreSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_SPINCTRL_UPDATED event handler for ID_SPINCTRL_NOTE_DURATION
    void OnSpinctrlNoteDurationUpdated( wxSpinEvent& event );

    /// wxEVT_COMMAND_CHECKBOX_CLICKED event handler for ID_CHECKBOX_MUTE_PREVIOUS_NOTE
    void OnCheckboxMutePreviousNoteClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHOICE_SELECTED event handler for ID_CHOICE_MUSICAL_INSTRUMENT
    void OnChoiceMusicalInstrumentSelected( wxCommandEvent& event );

////@end CVirtualInstrumentConfig event handler declarations

////@begin CVirtualInstrumentConfig member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CVirtualInstrumentConfig member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

	// Send a wxECVT_COMMAND_SLIDER_UPDATED event
	void UpdateSliderEvent(int id);

////@begin CVirtualInstrumentConfig member variables
    wxSlider* m_sldSensitivity;
    wxTextCtrl* m_txtSensorSensitivity;
    wxCheckBox* m_chkAutoDetectMotionArea;
    wxChoice* m_choMusicalScore;
    wxSpinCtrl* m_spinNoteDuration;
    wxCheckBox* m_chkMutePreviousNote;
    wxChoice* m_choMusicalInstrument;
////@end CVirtualInstrumentConfig member variables

	CVirtualInstrumentActivity* m_activity; 
};

#endif
    // _CVIRTUALINSTRUMENTCONFIG_H_
