/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2008 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#ifndef EXCEPTIONS__H
#define EXCEPTIONS__H

#include <wx/string.h>
#include <wx/intl.h>

class BaseException 
{
public:
	BaseException (wxString msg)
	{
		if (msg.Length()> 0)
			m_msg= msg;
		else
			SetDefaultMessage();
	}
	virtual void SetDefaultMessage ()
	{
		m_msg = _("General exception");
	}
	wxString& getMessage()
	{
		return m_msg;
	}
protected:
	wxString m_msg;
};

class InitializationException : public BaseException
{
public:
	InitializationException (wxString msg) : BaseException (msg) {}	
	virtual void SetDefaultMessage ()
	{
		m_msg = _("Error initializing system");
	}
};

class RunTimeException : public BaseException
{
public:
	RunTimeException (wxString msg) : BaseException (msg) {}	
	virtual void SetDefaultMessage ()
	{
		m_msg = _("Run time exception");
	}
};

class FileNotFoundException : public BaseException
{
public:
	FileNotFoundException (wxString msg) : BaseException (msg) {}	
	virtual void SetDefaultMessage ()
	{
		m_msg = _("File not found exception");
	}
};

class NoSuchObjectException : public BaseException
{
public:
	NoSuchObjectException (wxString msg) : BaseException (msg) {}	
	virtual void SetDefaultMessage ()
	{
		m_msg = _("No such object exception");
	}
};

class ValueOutOfRangeException : public BaseException
{
public:
	ValueOutOfRangeException (wxString msg) : BaseException (msg) {}	
	virtual void SetDefaultMessage ()
	{
		m_msg = _("Value Out Of Range Exception");
	}
};

class CameraException : public BaseException
{
public:
	CameraException (wxString msg) : BaseException (msg) {}	
	virtual void SetDefaultMessage ()
	{
		m_msg = _("Camera Exception");
	}
};

#endif